@@ -4,6 +4,8 @@
 package com.typesafe.config;
 
 import java.time.Duration;
+import java.time.Period;
+import java.time.temporal.TemporalAmount;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
@@ -34,7 +36,7 @@ import java.util.concurrent.TimeUnit;
  * 
  * <p>
  * You can find an example app and library <a
- * href="https://github.com/typesafehub/config/tree/master/examples">on
+ * href="https://github.com/lightbend/config/tree/master/examples">on
  * GitHub</a>. Also be sure to read the <a
  * href="package-summary.html#package_description">package overview</a> which
  * describes the big picture as shown in those examples.
@@ -54,7 +56,7 @@ import java.util.concurrent.TimeUnit;
  * in a JSON object; it's just a string that's the key in a map. A "path" is a
  * parseable expression with a syntax and it refers to a series of keys. Path
  * expressions are described in the <a
- * href="https://github.com/typesafehub/config/blob/master/HOCON.md">spec for
+ * href="https://github.com/lightbend/config/blob/master/HOCON.md">spec for
  * Human-Optimized Config Object Notation</a>. In brief, a path is
  * period-separated so "a.b.c" looks for key c in object b in object a in the
  * root object. Sometimes double quotes are needed around special characters in
@@ -108,7 +110,7 @@ import java.util.concurrent.TimeUnit;
  * <p>
  * <em>Substitutions</em> are the <code>${foo.bar}</code> syntax in config
  * files, described in the <a href=
- * "https://github.com/typesafehub/config/blob/master/HOCON.md#substitutions"
+ * "https://github.com/lightbend/config/blob/master/HOCON.md#substitutions"
  * >specification</a>. Resolving substitutions replaces these references with real
  * values.
  * 
@@ -187,7 +189,7 @@ public interface Config extends ConfigMergeable {
     /**
      * Returns a replacement config with all substitutions (the
      * <code>${foo.bar}</code> syntax, see <a
-     * href="https://github.com/typesafehub/config/blob/master/HOCON.md">the
+     * href="https://github.com/lightbend/config/blob/master/HOCON.md">the
      * spec</a>) resolved. Substitutions are looked up using this
      * <code>Config</code> as the root object, that is, a substitution
      * <code>${foo.bar}</code> will be replaced with the result of
@@ -668,7 +670,7 @@ public interface Config extends ConfigMergeable {
      * the value is already a number, then it's left alone; if it's a string,
      * it's parsed understanding unit suffixes such as "128K", as documented in
      * the <a
-     * href="https://github.com/typesafehub/config/blob/master/HOCON.md">the
+     * href="https://github.com/lightbend/config/blob/master/HOCON.md">the
      * spec</a>.
      *
      * @param path
@@ -688,7 +690,7 @@ public interface Config extends ConfigMergeable {
      * the value is already a number, then it's left alone; if it's a string,
      * it's parsed understanding unit suffixes such as "128K", as documented in
      * the <a
-     * href="https://github.com/typesafehub/config/blob/master/HOCON.md">the
+     * href="https://github.com/lightbend/config/blob/master/HOCON.md">the
      * spec</a>.
      *
      * @since 1.3.0
@@ -709,7 +711,7 @@ public interface Config extends ConfigMergeable {
      * Get value as a duration in milliseconds. If the value is already a
      * number, then it's left alone; if it's a string, it's parsed understanding
      * units suffixes like "10m" or "5ns" as documented in the <a
-     * href="https://github.com/typesafehub/config/blob/master/HOCON.md">the
+     * href="https://github.com/lightbend/config/blob/master/HOCON.md">the
      * spec</a>.
      *
      * @deprecated  As of release 1.1, replaced by {@link #getDuration(String, TimeUnit)}
@@ -752,7 +754,7 @@ public interface Config extends ConfigMergeable {
      * number, then it's taken as milliseconds and then converted to the
      * requested TimeUnit; if it's a string, it's parsed understanding units
      * suffixes like "10m" or "5ns" as documented in the <a
-     * href="https://github.com/typesafehub/config/blob/master/HOCON.md">the
+     * href="https://github.com/lightbend/config/blob/master/HOCON.md">the
      * spec</a>.
      * 
      * @since 1.2.0
@@ -776,7 +778,7 @@ public interface Config extends ConfigMergeable {
      * already a number, then it's taken as milliseconds; if it's
      * a string, it's parsed understanding units suffixes like
      * "10m" or "5ns" as documented in the <a
-     * href="https://github.com/typesafehub/config/blob/master/HOCON.md">the
+     * href="https://github.com/lightbend/config/blob/master/HOCON.md">the
      * spec</a>. This method never returns null.
      *
      * @since 1.3.0
@@ -793,6 +795,44 @@ public interface Config extends ConfigMergeable {
      */
     Duration getDuration(String path);
 
+    /**
+     * Gets a value as a java.time.Period. If the value is
+     * already a number, then it's taken as days; if it's
+     * a string, it's parsed understanding units suffixes like
+     * "10d" or "5w" as documented in the <a
+     * href="https://github.com/lightbend/config/blob/master/HOCON.md">the
+     * spec</a>. This method never returns null.
+     *
+     * @since 1.3.0
+     *
+     * @param path
+     *            path expression
+     * @return the period value at the requested path
+     * @throws ConfigException.Missing
+     *             if value is absent or null
+     * @throws ConfigException.WrongType
+     *             if value is not convertible to Long or String
+     * @throws ConfigException.BadValue
+     *             if value cannot be parsed as a number of the given TimeUnit
+     */
+    Period getPeriod(String path);
+
+    /**
+     * Gets a value as a java.time.temporal.TemporalAmount.
+     * This method will first try get get the value as a java.time.Duration, and if unsuccessful,
+     * then as a java.time.Period.
+     * This means that values like "5m" will be parsed as 5 minutes rather than 5 months
+     * @param path path expression
+     * @return the temporal value at the requested path
+     * @throws ConfigException.Missing
+     *             if value is absent or null
+     * @throws ConfigException.WrongType
+     *             if value is not convertible to Long or String
+     * @throws ConfigException.BadValue
+     *             if value cannot be parsed as a TemporalAmount
+     */
+    TemporalAmount getTemporal(String path);
+
     /**
      * Gets a list value (with any element type) as a {@link ConfigList}, which
      * implements {@code java.util.List<ConfigValue>}. Throws if the path is
