@@ -8,8 +8,11 @@ import com.typesafe.config.impl.Parseable;
 
 import java.io.File;
 import java.io.Reader;
+import java.lang.reflect.InvocationTargetException;
+import java.net.MalformedURLException;
 import java.net.URL;
 import java.util.Map;
+import java.util.Optional;
 import java.util.Properties;
 import java.util.concurrent.Callable;
 
@@ -28,7 +31,7 @@ import java.util.concurrent.Callable;
  * from a resource and nothing else.
  *
  * <p> You can find an example app and library <a
- * href="https://github.com/typesafehub/config/tree/master/examples">on
+ * href="https://github.com/lightbend/config/tree/master/examples">on
  * GitHub</a>.  Also be sure to read the <a
  * href="package-summary.html#package_description">package
  * overview</a> which describes the big picture as shown in those
@@ -36,6 +39,7 @@ import java.util.concurrent.Callable;
  */
 public final class ConfigFactory {
     private static final String STRATEGY_PROPERTY_NAME = "config.strategy";
+    private static final String OVERRIDE_WITH_ENV_PROPERTY_NAME = "config.override_with_env_vars";
 
     private ConfigFactory() {
     }
@@ -79,14 +83,14 @@ public final class ConfigFactory {
     /**
      * Like {@link #load(String)} but uses the supplied class loader instead of
      * the current thread's context class loader.
-     * 
+     *
      * <p>
      * To load a standalone resource (without the default reference and default
      * overrides), use {@link #parseResourcesAnySyntax(ClassLoader, String)}
      * rather than this method. To load only the reference config use
      * {@link #defaultReference(ClassLoader)} and to load only the overrides use
      * {@link #defaultOverrides(ClassLoader)}.
-     * 
+     *
      * @param loader class loader to look for resources in
      * @param resourceBasename basename (no .conf/.json/.properties suffix)
      * @return configuration for an application relative to given class loader
@@ -210,7 +214,8 @@ public final class ConfigFactory {
      * @return resolved configuration with overrides and fallbacks added
      */
     public static Config load(ClassLoader loader, Config config, ConfigResolveOptions resolveOptions) {
-        return defaultOverrides(loader).withFallback(config).withFallback(defaultReference(loader))
+        return defaultOverrides(loader).withFallback(config)
+                .withFallback(ConfigImpl.defaultReferenceUnresolved(loader))
                 .resolve(resolveOptions);
     }
 
@@ -329,27 +334,27 @@ public final class ConfigFactory {
      * by merging all resources "reference.conf" found on the classpath and
      * overriding the result with system properties. The returned reference
      * configuration will already have substitutions resolved.
-     * 
+     *
      * <p>
      * Libraries and frameworks should ship with a "reference.conf" in their
      * jar.
-     * 
+     *
      * <p>
      * The reference config must be looked up in the class loader that contains
      * the libraries that you want to use with this config, so the
      * "reference.conf" for each library can be found. Use
      * {@link #defaultReference(ClassLoader)} if the context class loader is not
      * suitable.
-     * 
+     *
      * <p>
      * The {@link #load()} methods merge this configuration for you
      * automatically.
-     * 
+     *
      * <p>
      * Future versions may look for reference configuration in more places. It
      * is not guaranteed that this method <em>only</em> looks at
      * "reference.conf".
-     * 
+     *
      * @return the default reference config for context class loader
      */
     public static Config defaultReference() {
@@ -367,6 +372,56 @@ public final class ConfigFactory {
         return ConfigImpl.defaultReference(loader);
     }
 
+    /**
+     * Obtains the default reference configuration, which is currently created
+     * by merging all resources "reference.conf" found on the classpath and
+     * overriding the result with system properties.
+     *
+     * <p>
+     * While the returned reference configuration is guaranteed to be
+     * resolvable (that is, there will be no substitutions that cannot be
+     * resolved), it is returned in an unresolved state for the purpose of
+     * allowing substitutions to be overridden by a config layer that falls
+     * back to this one.
+     *
+     * <p>
+     * Libraries and frameworks should ship with a "reference.conf" in their
+     * jar.
+     *
+     * <p>
+     * The reference config must be looked up in the class loader that contains
+     * the libraries that you want to use with this config, so the
+     * "reference.conf" for each library can be found. Use
+     * {@link #defaultReference(ClassLoader)} if the context class loader is not
+     * suitable.
+     *
+     * <p>
+     * The {@link #load()} methods merge this configuration for you
+     * automatically.
+     *
+     * <p>
+     * Future versions may look for reference configuration in more places. It
+     * is not guaranteed that this method <em>only</em> looks at
+     * "reference.conf".
+     *
+     * @return the unresolved default reference config for the context class
+     *         loader
+     */
+    public static Config defaultReferenceUnresolved() {
+        return defaultReferenceUnresolved(checkedContextClassLoader("defaultReferenceUnresolved"));
+    }
+
+    /**
+     * Like {@link #defaultReferenceUnresolved()} but allows you to specify a
+     * class loader to use rather than the current context class loader.
+     *
+     * @param loader class loader to look for resources in
+     * @return the unresolved default reference config for this class loader
+     */
+    public static Config defaultReferenceUnresolved(ClassLoader loader) {
+        return ConfigImpl.defaultReferenceUnresolved(loader);
+    }
+
     /**
      * Obtains the default override configuration, which currently consists of
      * system properties. The returned override configuration will already have
@@ -383,7 +438,11 @@ public final class ConfigFactory {
      * @return the default override configuration
      */
     public static Config defaultOverrides() {
-        return systemProperties();
+        if (getOverrideWithEnv()) {
+            return systemEnvironmentOverrides().withFallback(systemProperties());
+        } else {
+            return systemProperties();
+        }
     }
 
     /**
@@ -394,7 +453,7 @@ public final class ConfigFactory {
      * @return the default override configuration
      */
     public static Config defaultOverrides(ClassLoader loader) {
-        return systemProperties();
+        return defaultOverrides();
     }
 
     /**
@@ -496,6 +555,8 @@ public final class ConfigFactory {
         // We rely on this having the side effect that it drops
         // all caches
         ConfigImpl.reloadSystemPropertiesConfig();
+        ConfigImpl.reloadEnvVariablesConfig();
+        ConfigImpl.reloadEnvVariablesOverridesConfig();
     }
 
     /**
@@ -548,6 +609,50 @@ public final class ConfigFactory {
         return ConfigImpl.systemPropertiesAsConfig();
     }
 
+    /**
+     * Gets a <code>Config</code> containing the system's environment variables
+     * used to override configuration keys.
+     * Environment variables taken in considerations are starting with
+     * {@code CONFIG_FORCE_}
+     *
+     * <p>
+     * Environment variables are mangled in the following way after stripping the prefix "CONFIG_FORCE_":
+     * <table border="1">
+     * <tr>
+     *     <th bgcolor="silver">Env Var</th>
+     *     <th bgcolor="silver">Config</th>
+     * </tr>
+     * <tr>
+     *     <td>_&nbsp;&nbsp;&nbsp;[1 underscore]</td>
+     *     <td>. [dot]</td>
+     * </tr>
+     * <tr>
+     *     <td>__&nbsp;&nbsp;[2 underscore]</td>
+     *     <td>- [dash]</td>
+     *  </tr>
+     * <tr>
+     *     <td>___&nbsp;[3 underscore]</td>
+     *     <td>_ [underscore]</td>
+     * </tr>
+     * </table>
+     *
+     * <p>
+     * A variable like: {@code CONFIG_FORCE_a_b__c___d}
+     * is translated to a config key: {@code a.b-c_d}
+     *
+     * <p>
+     * This method can return a global immutable singleton, so it's preferred
+     * over parsing system properties yourself.
+     * <p>
+     * {@link #defaultOverrides} will include the system environment variables as
+     * overrides if `config.override_with_env_vars` is set to `true`.
+     *
+     * @return system environment variable overrides parsed into a <code>Config</code>
+     */
+    public static Config systemEnvironmentOverrides() {
+        return ConfigImpl.envVariablesOverridesAsConfig();
+    }
+
     /**
      * Gets a <code>Config</code> containing the system's environment variables.
      * This method can return a global immutable singleton.
@@ -569,7 +674,7 @@ public final class ConfigFactory {
     /**
      * Converts a Java {@link java.util.Properties} object to a
      * {@link ConfigObject} using the rules documented in the <a
-     * href="https://github.com/typesafehub/config/blob/master/HOCON.md">HOCON
+     * href="https://github.com/lightbend/config/blob/master/HOCON.md">HOCON
      * spec</a>. The keys in the <code>Properties</code> object are split on the
      * period character '.' and treated as paths. The values will all end up as
      * string values. If you have both "a=foo" and "a.b=bar" in your properties
@@ -988,6 +1093,89 @@ public final class ConfigFactory {
         return parseResourcesAnySyntax(resourceBasename, ConfigParseOptions.defaults());
     }
 
+    /**
+     * Parse only any application replacement (specified by one of config.{resource,file,url}), returning
+     * an empty Config if no overrides were set.
+     *
+     * @since 1.4.1
+     *
+     * @return a {@link java.util.Optional} containing any specified replacement, or {@link Optional#empty()}
+     * if none was specified.
+     */
+    public static java.util.Optional<Config> parseApplicationReplacement() {
+        return parseApplicationReplacement(ConfigParseOptions.defaults());
+    }
+
+    /**
+     * Like {@link #parseApplicationReplacement()} but allows you to specify a class loader
+     * ti yse rather than the current context class loader.
+     *
+     * @since 1.4.1
+     *
+     * @param loader the class loader
+     * @return a {@link java.util.Optional} containing any specified replacement, or {@link Optional#empty()}
+     * if none was specified.
+     */
+    public static java.util.Optional<Config> parseApplicationReplacement(ClassLoader loader) {
+        return parseApplicationReplacement(ConfigParseOptions.defaults().setClassLoader(loader));
+    }
+
+    /**
+     * Like {@link #parseApplicationReplacement()} but allows you to specify parse options.
+     *
+     * @since 1.4.1
+     *
+     * @param parseOptions parse options
+     * @return a {@link java.util.Optional} containing any specified replacement, or {@link Optional#empty()}
+     * if none was specified.
+     */
+    public static java.util.Optional<Config> parseApplicationReplacement(ConfigParseOptions parseOptions) {
+        final ConfigParseOptions withLoader = ensureClassLoader(parseOptions, "parseApplicationReplacement");
+        ClassLoader loader = withLoader.getClassLoader();
+
+        int specified = 0;
+
+        // override application.conf with config.file, config.resource,
+        // config.url if requested.
+        String resource = System.getProperty("config.resource");
+        if (resource != null)
+            specified += 1;
+        String file = System.getProperty("config.file");
+        if (file != null)
+            specified += 1;
+        String url = System.getProperty("config.url");
+        if (url != null)
+            specified += 1;
+
+        if (specified == 0) {
+            return java.util.Optional.empty();
+        } else if (specified > 1) {
+            throw new ConfigException.Generic("You set more than one of config.file='" + file
+                + "', config.url='" + url + "', config.resource='" + resource
+                + "'; don't know which one to use!");
+        } else {
+            // the override file/url/resource MUST be present or it's an error
+            ConfigParseOptions overrideOptions = parseOptions.setAllowMissing(false);
+            if (resource != null) {
+                if (resource.startsWith("/"))
+                    resource = resource.substring(1);
+                // this deliberately does not parseResourcesAnySyntax; if
+                // people want that they can use an include statement.
+                return java.util.Optional.of(ConfigFactory.parseResources(loader, resource, overrideOptions));
+            } else if (file != null) {
+                return java.util.Optional.of(ConfigFactory.parseFile(new File(file), overrideOptions));
+            } else {
+                try {
+                    return java.util.Optional.of(ConfigFactory.parseURL(new URL(url), overrideOptions));
+                } catch (MalformedURLException e) {
+                    throw new ConfigException.Generic("Bad URL in config.url system property: '"
+                        + url + "': " + e.getMessage(), e);
+                }
+            }
+        }
+
+    }
+
     /**
      * Parses a string (which should be valid HOCON or JSON by default, or
      * the syntax specified in the options otherwise).
@@ -1054,7 +1242,11 @@ public final class ConfigFactory {
 
         if (className != null) {
             try {
-                return ConfigLoadingStrategy.class.cast(Class.forName(className).newInstance());
+                return Class.forName(className).asSubclass(ConfigLoadingStrategy.class).getDeclaredConstructor().newInstance();
+            } catch (InvocationTargetException e) {
+                Throwable cause = e.getCause();
+                if (cause == null) throw new ConfigException.BugOrBroken("Failed to load strategy: " + className, e);
+                else throw new ConfigException.BugOrBroken("Failed to load strategy: " + className, cause);
             } catch (Throwable e) {
                 throw new ConfigException.BugOrBroken("Failed to load strategy: " + className, e);
             }
@@ -1062,4 +1254,10 @@ public final class ConfigFactory {
             return new DefaultConfigLoadingStrategy();
         }
     }
+
+    private static Boolean getOverrideWithEnv() {
+        String overrideWithEnv = System.getProperties().getProperty(OVERRIDE_WITH_ENV_PROPERTY_NAME);
+
+        return Boolean.parseBoolean(overrideWithEnv);
+    }
 }
